﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
// -------- ▼for 共有ターゲット。
//-using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.DataTransfer;
using Windows.ApplicationModel.DataTransfer.ShareTarget;

// 空のアプリケーション テンプレートについては、http://go.microsoft.com/fwlink/?LinkId=234227 を参照してください

namespace PerfectCS_2012
{
    /// <summary>
    /// 既定の Application クラスを補完するアプリケーション固有の動作を提供します。
    /// </summary>
    sealed partial class App : Application
    {
        /// <summary>
        /// 単一アプリケーション オブジェクトを初期化します。これは、実行される作成したコードの
        /// 最初の行であり、main() または WinMain() と論理的に等価です。
        /// </summary>
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;

        }

        // -------- ▼for 共有ターゲット。
        protected override async void OnShareTargetActivated(ShareTargetActivatedEventArgs args)
        {
            ShareOperation shareOperation = args.ShareOperation;
            DataPackageView dataPackageView = shareOperation.Data;
            if (dataPackageView.Contains(StandardDataFormats.Uri))
            {
                Uri uri = await dataPackageView.GetUriAsync();
                if (uri != null)
                {
                    BasicPage1.TargetUrl = uri.AbsoluteUri;
                    // ---- ※対象URLの設定。

                    // ---- ※実行中であれば、共有ターゲットの反映。
                    if (args.PreviousExecutionState == Windows.ApplicationModel.Activation.ApplicationExecutionState.Running)
                    {
                        Frame frame = Window.Current.Content as Frame;
                        if (frame != null)
                        {
                            frame.Navigate(typeof(BasicPage1));
                        }
                    }
                }
            }
#if false
            shareOperation.ReportCompleted();
#endif
        }

        /// <summary>
        /// アプリケーションがエンド ユーザーによって正常に起動されたときに呼び出されます。他のエントリ ポイントは、
        /// アプリケーションが特定のファイルを開くために呼び出されたときに
        /// 検索結果やその他の情報を表示するために使用されます。
        /// </summary>
        /// <param name="args">起動要求とプロセスの詳細を表示します。</param>
        // ---- ▼async：非同期メソッドに。
        protected async override void OnLaunched(LaunchActivatedEventArgs args)
        {
            Frame rootFrame = Window.Current.Content as Frame;

            // ウィンドウに既にコンテンツが表示されている場合は、アプリケーションの初期化を繰り返さずに、
            // ウィンドウがアクティブであることだけを確認してください
            if (rootFrame == null)
            {
                // ナビゲーション コンテキストとして動作するフレームを作成し、最初のページに移動します
                rootFrame = new Frame();

                // ---- ▼SuspensionManagerにFrameを登録。
                string sessionStateKey = "rootFrame";
                PerfectCS_2012.Common.SuspensionManager.RegisterFrame(rootFrame, sessionStateKey);
                // ----

                if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    //TODO: 以前中断したアプリケーションから状態を読み込みます。
                    // ---- ▼SuspensionManagerに登録されているFrameのナビゲーション状態を復元。
                    await PerfectCS_2012.Common.SuspensionManager.RestoreAsync();
                }

                // フレームを現在のウィンドウに配置します
                Window.Current.Content = rootFrame;
            }

            if (rootFrame.Content == null)
            {
                // ナビゲーション スタックが復元されていない場合、最初のページに移動します。
                // このとき、必要な情報をナビゲーション パラメーターとして渡して、新しいページを
                // 構成します

                // ---- ▼BasicPage1へ遷移。
                if (!rootFrame.Navigate(typeof(BasicPage1), args.Arguments))
                //-if (!rootFrame.Navigate(typeof(BasicPage2), args.Arguments))
                //-if (!rootFrame.Navigate(typeof(BasicPage3), args.Arguments))
                {
                    throw new Exception("Failed to create initial page");
                }
            }
            // 現在のウィンドウがアクティブであることを確認します
            Window.Current.Activate();
        }

        /// <summary>
        /// アプリケーションの実行が中断されたときに呼び出されます。アプリケーションの状態は、
        /// アプリケーションが終了されるのか、メモリの内容がそのままで再開されるのか
        /// わからない状態で保存されます。
        /// </summary>
        /// <param name="sender">中断要求の送信元。</param>
        /// <param name="e">中断要求の詳細。</param>
        // ---- ▼async：非同期メソッドに。
        private async void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            //TODO: アプリケーションの状態を保存してバックグラウンドの動作があれば停止します
            // ---- ▼SuspensionManagerに登録されているFrameのナビゲーション状態を保存。
            await PerfectCS_2012.Common.SuspensionManager.SaveAsync();

            deferral.Complete();
        }

        /// <summary>
        /// 検索結果を表示するためにアプリケーションがアクティブになるときに呼び出されます。
        /// </summary>
        /// <param name="args">アクティブ化要求に関する詳細を表示します。</param>
        protected async override void OnSearchActivated(Windows.ApplicationModel.Activation.SearchActivatedEventArgs args)
        {
            // TODO: アプリケーションが既に実行されている場合は、検索時間を短縮するために OnWindowCreated で
            // Windows.ApplicationModel.Search.SearchPane.GetForCurrentView().QuerySubmitted イベントを登録します

            // ウィンドウで Frame ナビゲーションがまだ使用されていない場合は、独自の Frame を挿入します
            var previousContent = Window.Current.Content;
            var frame = previousContent as Frame;

            // アプリケーションにトップレベルのフレームが含まれていない場合は、これが 
            // 最初に起動される可能性があります。通常は、App.xaml.cs のこのメソッドおよび 
            // OnLaunched で共通のメソッドを呼び出すことができます。
            if (frame == null)
            {
                // ナビゲーション コンテキストとして動作するフレームを作成し、
                // SuspensionManager キーに関連付けます
                frame = new Frame();
                PerfectCS_2012.Common.SuspensionManager.RegisterFrame(frame, "AppFrame");

                if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    // 必要な場合のみ、保存されたセッション状態を復元します
                    try
                    {
                        await PerfectCS_2012.Common.SuspensionManager.RestoreAsync();
                    }
                    catch (PerfectCS_2012.Common.SuspensionManagerException)
                    {
                        // 状態の復元に何か問題があります。
                        // 状態がないものとして続行します
                    }
                }
            }

            frame.Navigate(typeof(SearchResultsPage1), args.QueryText);
            Window.Current.Content = frame;

            // 現在のウィンドウがアクティブであることを確認します
            Window.Current.Activate();
        }
    }
}
