﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
// --------
using System.Runtime.CompilerServices;
// --------
namespace PerfectCS_2012
{
    class Exapmle_LINQtoEntities
    {
        public Exapmle_LINQtoEntities()
        {
            m_container = new ReceiptModelContainer();
            // ----
            //-m_container.ObjectContext;

        }
        // --------
        private ReceiptModelContainer m_container;

        // -------- Caller属性
        public void TraceMessage(string message,
            [CallerMemberName] string callerMemberName = "",
            [CallerFilePath] string callerFilePath = "",
            [CallerLineNumber] int callerLineNumber = 0)
        {
            Console.WriteLine("message = {0}", message);
            Console.WriteLine("caller Member = {0} / File = {1} / Line = {2}",
                callerMemberName, callerFilePath, callerLineNumber);
        }

        // -------- foreach文の要素変数のラムダ式における参照
        public void ForeachTest()
        {
            string[] texts = new string[] { "a", "b", "c" };
            Action action = null;
            foreach (string text in texts)
            {
                action += () => Console.WriteLine(text);
            }
            action();
        }
        // -------- 選択（抽出）
        public void Select(decimal borderAmount)
        {
            Console.WriteLine("Select (Border Amount = {0})", borderAmount);
            // ----
            //-IEnumerable<Receipt>
            System.Linq.IQueryable<Receipt>
            //-var 
            query = from receipt in m_container.Receipts
                           where receipt.Amount > borderAmount
                           select receipt;
            // ----
            string sql = query.ToString();
            Console.WriteLine("\nSQL = {0}", sql);
            // ----
            string databaseText = m_container.Database.ToString();
            Console.WriteLine("DatabaseText = {0}", databaseText);
            // ----
            foreach (var receipt in query)
            {
                Console.WriteLine("\n____Receipt[{0}] {1:yyyy/MM/dd} - {2} {3}",
                    receipt.ID,
                    receipt.Date,
                    receipt.Title,
                    receipt.Amount);
                // ----
                foreach (var receiptDetail in receipt.ReceiptDetails)
                {
                    Console.WriteLine("    ____Detail[{0}] {1}({2}) * {3} = {4}",
                        receiptDetail.ID,
                        receiptDetail.Name,
                        receiptDetail.Value,
                        receiptDetail.Quantity,
                        receiptDetail.Price);
                }
            }
        }

        // -------- 挿入
        public void Insert(string[] receiptTitles, ReceiptDetail[] shareReceiptDetails, int receiptDetailCount)
        {
            Console.WriteLine("Insert (Receipt Count = {0})", receiptTitles.Length);
            // ----
            DateTime receiptDate = DateTime.Today;
            int detailIndex = 0;
            foreach (string receiptTitle in receiptTitles)
            {
                Receipt receipt = new Receipt()
                {
                    Date = receiptDate,
                    Title = receiptTitle
                };
                // ----
                decimal amount = 0;
                int detailCount = 0;
                while (detailCount < receiptDetailCount)
                {
                    ReceiptDetail shareReceiptDetail = shareReceiptDetails[detailIndex];
                    ReceiptDetail receiptDetail = new ReceiptDetail()
                    {
                        Name = shareReceiptDetail.Name,
                        Value = shareReceiptDetail.Value,
                        Quantity = shareReceiptDetail.Quantity
                    };
                    // ----
                    receiptDetail.Price = receiptDetail.Value * receiptDetail.Quantity;
#if true
                    receipt.ReceiptDetails.Add(receiptDetail);
#else
                    receiptDetail.Receipt = receipt;    // -- ※
                    m_container.ReceiptDetails.Add(receiptDetail);
#endif
                    // ----
                    amount += receiptDetail.Price;
                    detailIndex++;
                    if (detailIndex >= shareReceiptDetails.Length)
                    {
                        detailIndex = 0;
                    }
                    detailCount++;
                }
                receipt.Amount = amount;
                m_container.Receipts.Add(receipt);
            }
            // ----
            m_container.SaveChanges();
        }


        // -------- 更新
        public void Update(decimal borderAmount, string maxPriceMark)
        {
            Console.WriteLine("Update (Border Amount = {0}, Max Price Mark = {1})", borderAmount, maxPriceMark);
            // ----
            var query = from receipt in m_container.Receipts
                           where receipt.Amount > borderAmount
                           select receipt;
            // ----
            foreach (Receipt receipt in query)
            {
                ReceiptDetail maxPriceReceiptDetail = null;
                foreach (ReceiptDetail receiptDetail in receipt.ReceiptDetails)
                {
                    if (maxPriceReceiptDetail == null ||
                        maxPriceReceiptDetail.Price < receiptDetail.Price)
                    {
                        maxPriceReceiptDetail = receiptDetail;
                    }
                }
                // ----
                if (maxPriceReceiptDetail != null)
                {
                    maxPriceReceiptDetail.Name = maxPriceMark + maxPriceReceiptDetail.Name;
                }
            }
            // ----
            m_container.SaveChanges();
        }

        // -------- 削除
        public void Delete(decimal borderAmount)
        {
            Console.WriteLine("Delete (Border Amount = {0})", borderAmount);
            // ----
            var query = from receipt in m_container.Receipts
                           where receipt.Amount > borderAmount
                           select receipt;
            // ----
            foreach (Receipt receipt in query)
            {
                foreach (ReceiptDetail receiptDetail in receipt.ReceiptDetails.ToArray())
                {
#if false
                    receipt.ReceiptDetails.Remove(receiptDetail);
#else
                    m_container.ReceiptDetails.Remove(receiptDetail);
#endif
                }
                // ----
                m_container.Receipts.Remove(receipt);
            }
            // ----
            m_container.SaveChanges();
        }

    }
}
