﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PerfectCS_2012
{
    public class ConsoleTrace
    {
        // --------
        // <caption>=[_value]
        public static void WriteValue(string caption, object _value)
        {
            string traceLine = caption;
            // ----
            traceLine += " = [";
            if (_value != null)
            {
                traceLine += _value.ToString();
            }
            traceLine += "]";
            // ----
            Console.WriteLine(traceLine);
        }
        // --------
        // <caption>[<index>]=[_value]
        public static void WriteValue(string caption, int? index, object _value)
        {
            if (index != null)
            {
                caption += "[";
                caption += index.ToString();
                caption += "]";
            }
            // ----
            ConsoleTrace.WriteValue(caption, _value);
        }
        // --------
        // <caption>[0]=[value0]
        // <caption>[1]=[value1]
        // <caption>[2]=[value2]
        // ...
        public static void WriteValueList(string caption, params object[] values)
        {
            int index = 0;
            while (index < values.Length)
            {
                ConsoleTrace.WriteValue(caption, index, values[index]);
                index++;
            }
        }
        // --------
        // <caption>[<seq>]=[value0, value1, valie2, ...]
        public static void WriteValueList(string caption, int? seq, params object[] values)
        {
            string valuesList = "";
            int index = 0;
            while (index < values.Length)
            {
                if (index > 0)
                {
                    valuesList += ", ";
                }
                object value = values[index];
                if (value != null)
                {
                    valuesList += value.ToString();
                }
                index++;
            }
            ConsoleTrace.WriteValue(caption, seq, valuesList);
        }
        // --------
        // <caption>=[X, X, X, ...]
        public static void WriteChars(string caption, char[] chars)
        {
            string charList = "";
            int index = 0;
            while (index < chars.Length)
            {
                if (index > 0)
                {
                    charList += ", ";
                }
                if (chars[index] >= 0x20)
                {
                    charList += chars[index];
                }
                else
                {
                    byte _byte = (byte)chars[index];
                    charList += _byte.ToString("X2");
                }
                index++;
            }
            ConsoleTrace.WriteValue(caption, charList);
        }
        // --------
        // <caption>=[XX, XX, XX, ...]
        public static void WriteBytes(string caption, byte[] bytes)
        {
            string hexaList = "";
            int index = 0;
            while (index < bytes.Length)
            {
                if (index > 0)
                {
                    hexaList += ", ";
                }
                hexaList += bytes[index].ToString("X2");
                index++;
            }
            ConsoleTrace.WriteValue(caption, hexaList);
        }
    }
}
